# Copyright 2015 Oursky Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from .console import ConsoleTransport
from .http import HttpTransport

try:
    from .zmq import ZmqTransport
except ImportError:
    from .common import CommonTransport

    class ZmqTransport(CommonTransport):
        """A dummy ZmqTransport class to raise a proper exception
        """

        def __init__(self, addr, context=None, registry=None):
            raise ImportError(
                'zmq transport is not installed. '
                'Please install via `pip install --upgrade skygear[zmq]`')

__all__ = ['ConsoleTransport', 'HttpTransport', 'ZmqTransport']
